<?php

namespace Files\File\Policies;

use Litepie\User\Interfaces\UserPolicyInterface;
use Files\File\Models\File;

class FilePolicy
{

    /**
     * Determine if the given user can view the file.
     *
     * @param UserPolicyInterface $authUser
     * @param File $file
     *
     * @return bool
     */
    public function view(UserPolicyInterface $authUser, File $file)
    {
        if ($authUser->canDo('file.file.view') && $authUser->isAdmin()) {
            return true;
        }

        return $file->user_id == user_id() && $file->user_type == user_type();
    }

    /**
     * Determine if the given user can create a file.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function create(UserPolicyInterface $authUser)
    {
        return  $authUser->canDo('file.file.create');
    }

    /**
     * Determine if the given user can update the given file.
     *
     * @param UserPolicyInterface $authUser
     * @param File $file
     *
     * @return bool
     */
    public function update(UserPolicyInterface $authUser, File $file)
    {
        if ($authUser->canDo('file.file.edit') && $authUser->isAdmin()) {
            return true;
        }

        return $file->user_id == user_id() && $file->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given file.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function destroy(UserPolicyInterface $authUser, File $file)
    {
        return $file->user_id == user_id() && $file->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given file.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function verify(UserPolicyInterface $authUser, File $file)
    {
        if ($authUser->canDo('file.file.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given file.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function approve(UserPolicyInterface $authUser, File $file)
    {
        if ($authUser->canDo('file.file.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $authUser    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($authUser, $ability)
    {
        if ($authUser->isSuperuser()) {
            return true;
        }
    }
}
